@@ -1,6 +1,8 @@
 package diff2
 
 import (
+	"bytes"
+	"fmt"
 	"reflect"
 	"strings"
 	"testing"
@@ -18,6 +20,55 @@ func init() {
 	color.NoColor = true
 }
 
+// Stringify the datastructures (for debugging)
+
+func (c Change) String() string {
+	var buf bytes.Buffer
+	b := &buf
+
+	fmt.Fprintf(b, "Change: verb=%v\n", c.Type)
+	fmt.Fprintf(b, "    key=%v\n", c.Key)
+	if c.HintOnlyTTL {
+		fmt.Fprint(b, "    Hints=OnlyTTL\n", c.Key)
+	}
+	if len(c.Old) != 0 {
+		fmt.Fprintf(b, "    old=%v\n", c.Old)
+	}
+	if len(c.New) != 0 {
+		fmt.Fprintf(b, "    new=%v\n", c.New)
+	}
+	fmt.Fprintf(b, "    msg=%q\n", c.Msgs)
+
+	return b.String()
+}
+
+func (cl ChangeList) String() string {
+	var buf bytes.Buffer
+	b := &buf
+
+	fmt.Fprintf(b, "ChangeList: len=%d\n", len(cl))
+	for i, j := range cl {
+		fmt.Fprintf(b, "%02d: %s", i, j)
+	}
+
+	return b.String()
+}
+
+// Make sample data
+
+func makeRec(label, rtype, content string) *models.RecordConfig {
+	origin := "f.com"
+	r := models.RecordConfig{TTL: 300}
+	r.SetLabel(label, origin)
+	r.PopulateFromString(rtype, content, origin)
+	return &r
+}
+func makeRecTTL(label, rtype, content string, ttl uint32) *models.RecordConfig {
+	r := makeRec(label, rtype, content)
+	r.TTL = ttl
+	return r
+}
+
 var testDataAA1234 = makeRec("laba", "A", "1.2.3.4")               // [ 0]
 var testDataAA5678 = makeRec("laba", "A", "5.6.7.8")               //
 var testDataAA1234ttl700 = makeRecTTL("laba", "A", "1.2.3.4", 700) //
@@ -52,6 +103,11 @@ var d12 = makeRec("labh", "A", "1.2.3.4")                 // [12']
 var d13 = makeRec("labc", "CNAME", "labe")                // [13']
 var testDataApexMX22bbb = makeRec("", "MX", "22 bbb")
 
+func justMsgString(cl ChangeList) string {
+	msgs := justMsgs(cl)
+	return strings.Join(msgs, "\n")
+}
+
 func compareMsgs(t *testing.T, fnname, testname, testpart string, gotcc ChangeList, wantstring string, wantstringdefault string) {
 	wantstring = coalesce(wantstring, wantstringdefault)
 	t.Helper()
