@@ -7,7 +7,6 @@ import (
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/providers"
 
@@ -42,17 +41,22 @@ func NewGCore(m map[string]string, metadata json.RawMessage) (providers.DNSServi
 }
 
 var features = providers.DocumentationNotes{
-	providers.CanAutoDNSSEC:          providers.Cannot(),
+	// The default for unlisted capabilities is 'Cannot'.
+	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanAutoDNSSEC:          providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanUseAlias:            providers.Cannot(),
+	providers.CanConcur:              providers.Cannot(),
+	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Cannot(),
 	providers.CanUseLOC:              providers.Cannot(),
 	providers.CanUseNAPTR:            providers.Cannot(),
-	providers.CanUsePTR:              providers.Cannot(),
+	providers.CanUsePTR:              providers.Can("G-Core supports PTR records only in rDNS zones"),
 	providers.CanUseSRV:              providers.Can("G-Core doesn't support SRV records with empty targets"),
 	providers.CanUseSSHFP:            providers.Cannot(),
 	providers.CanUseTLSA:             providers.Cannot(),
+	providers.CanUseHTTPS:            providers.Can(),
+	providers.CanUseSVCB:             providers.Can(),
 	providers.DocCreateDomains:       providers.Can(),
 	providers.DocDualHost:            providers.Can(),
 	providers.DocOfficiallySupported: providers.Cannot(),
@@ -137,120 +141,79 @@ func (c *gcoreProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exist
 	var deletions []*models.Correction
 	var reports []*models.Correction
 
-	if !diff2.EnableDiff2 {
-
-		// diff existing vs. current.
-		differ := diff.New(dc)
-		keysToUpdate, err := differ.ChangedGroups(existing)
-		if err != nil {
-			return nil, err
+	// Gcore auto uses ALIAS for apex zone CNAME records, just like CloudFlare
+	for _, rec := range dc.Records {
+		if rec.Type == "ALIAS" {
+			rec.Type = "CNAME"
 		}
-		if len(keysToUpdate) == 0 {
-			return nil, nil
+	}
+
+	changes, err := diff2.ByRecordSet(existing, dc, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	for _, change := range changes {
+		record := recordsToNative(change.New, change.Key)
+
+		// Copy all params to avoid overwrites
+		zone := dc.Name
+		name := change.Key.NameFQDN
+		typ := change.Key.Type
+		msg := generateChangeMsg(change.Msgs)
+
+		switch change.Type {
+		case diff2.REPORT:
+			corrections = append(corrections, &models.Correction{Msg: change.MsgsJoined})
+		case diff2.CREATE:
+			corrections = append(corrections, &models.Correction{
+				Msg: msg,
+				F: func() error {
+					return c.provider.CreateRRSet(c.ctx, zone, name, typ, *record)
+				},
+			})
+		case diff2.CHANGE:
+			corrections = append(corrections, &models.Correction{
+				Msg: msg,
+				F: func() error {
+					return c.provider.UpdateRRSet(c.ctx, zone, name, typ, *record)
+				},
+			})
+		case diff2.DELETE:
+			deletions = append(deletions, &models.Correction{
+				Msg: msg,
+				F: func() error {
+					return c.provider.DeleteRRSet(c.ctx, zone, name, typ)
+				},
+			})
+		default:
+			panic(fmt.Sprintf("unhandled change.Type %s", change.Type))
 		}
+	}
 
-		desiredRecords := dc.Records.GroupedByKey()
-		existingRecords := existing.GroupedByKey()
+	dnssecEnabled, err := c.dnssdkGetDNSSEC(dc.Name)
+	if err != nil {
+		return nil, err
+	}
 
-		for label := range keysToUpdate {
-			if _, ok := desiredRecords[label]; !ok {
-				// record deleted in update
-				// Copy all params to avoid overwrites
-				zone := dc.Name
-				name := label.NameFQDN
-				typ := label.Type
-				msg := generateChangeMsg(keysToUpdate[label])
-				deletions = append(deletions, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.DeleteRRSet(c.ctx, zone, name, typ)
-					},
-				})
-
-			} else if _, ok := existingRecords[label]; !ok {
-				// record created in update
-				record := recordsToNative(desiredRecords[label], label)
-				if record == nil {
-					panic("No records matching label")
-				}
-
-				// Copy all params to avoid overwrites
-				zone := dc.Name
-				name := label.NameFQDN
-				typ := label.Type
-				msg := generateChangeMsg(keysToUpdate[label])
-				corrections = append(corrections, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.CreateRRSet(c.ctx, zone, name, typ, *record)
-					},
-				})
-
-			} else {
-				// record modified in update
-				record := recordsToNative(desiredRecords[label], label)
-				if record == nil {
-					panic("No records matching label")
-				}
-
-				// Copy all params to avoid overwrites
-				zone := dc.Name
-				name := label.NameFQDN
-				typ := label.Type
-				msg := generateChangeMsg(keysToUpdate[label])
-				corrections = append(corrections, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.UpdateRRSet(c.ctx, zone, name, typ, *record)
-					},
-				})
-			}
-		}
-
-	} else {
-		// Diff2 version
-		changes, err := diff2.ByRecordSet(existing, dc, nil)
-		if err != nil {
-			return nil, err
-		}
-
-		for _, change := range changes {
-			record := recordsToNative(change.New, change.Key)
-
-			// Copy all params to avoid overwrites
-			zone := dc.Name
-			name := change.Key.NameFQDN
-			typ := change.Key.Type
-			msg := generateChangeMsg(change.Msgs)
-
-			switch change.Type {
-			case diff2.REPORT:
-				corrections = append(corrections, &models.Correction{Msg: change.MsgsJoined})
-			case diff2.CREATE:
-				corrections = append(corrections, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.CreateRRSet(c.ctx, zone, name, typ, *record)
-					},
-				})
-			case diff2.CHANGE:
-				corrections = append(corrections, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.UpdateRRSet(c.ctx, zone, name, typ, *record)
-					},
-				})
-			case diff2.DELETE:
-				deletions = append(deletions, &models.Correction{
-					Msg: msg,
-					F: func() error {
-						return c.provider.DeleteRRSet(c.ctx, zone, name, typ)
-					},
-				})
-			default:
-				panic(fmt.Sprintf("unhandled change.Type %s", change.Type))
-			}
-		}
+	if !dnssecEnabled && dc.AutoDNSSEC == "on" {
+		// Copy all params to avoid overwrites
+		zone := dc.Name
+		corrections = append(corrections, &models.Correction{
+			Msg: "Enable DNSSEC",
+			F: func() error {
+				return c.dnssdkSetDNSSEC(zone, true)
+			},
+		})
+	} else if dnssecEnabled && dc.AutoDNSSEC == "off" {
+		// Copy all params to avoid overwrites
+		zone := dc.Name
+		corrections = append(corrections, &models.Correction{
+			Msg: "Disable DNSSEC",
+			F: func() error {
+				return c.dnssdkSetDNSSEC(zone, false)
+			},
+		})
 	}
 
 	result := append(reports, deletions...)
